"""Schemas for form input and validation"""

import colander
import deform

from ..models import UserRole
from .base import CSRFSchema

ROLES = [(role.name, role.name.capitalize()) for role in UserRole]


@colander.deferred
def deferred_unique_username_validator(node, kw):  # noqa: ARG001
    """checks if an username is not registered already"""

    def validate_unique_username(node, value):
        request = kw.get("request")
        try:
            request.repo.get_user_by_username(value)
            raise colander.Invalid(node, "User name already registered")
        except StopIteration:
            pass

    return validate_unique_username


@colander.deferred
def deferred_unique_email_validator(node, kw):  # noqa: ARG001
    """checks if an email is not registered already"""
    email_validator = colander.Email()

    def validate_unique_email(node, value):
        email_validator(node, value)  # raises exception on invalid address
        request = kw.get("request")
        try:
            request.repo.get_user_by_email(value)
            raise colander.Invalid(node, "Email address in use")
        except StopIteration:
            pass

    return validate_unique_email


class RegistrationSchema(CSRFSchema):
    """new user registration"""

    user_name = colander.SchemaNode(
        colander.String(),
        widget=deform.widget.TextInputWidget(
            template="textinput_disabled.pt", css_class="o3-reg-username"
        ),
        description="automagically generated for you",
        validator=deferred_unique_username_validator,
    )
    first_name = colander.SchemaNode(
        colander.String(),
        widget=deform.widget.TextInputWidget(
            css_class="o3-reg-firstname o3-reg-username-source"
        ),
    )
    last_name = colander.SchemaNode(
        colander.String(),
        widget=deform.widget.TextInputWidget(
            css_class="o3-reg-lastname o3-reg-username-source"
        ),
    )
    email = colander.SchemaNode(
        colander.String(),
        validator=deferred_unique_email_validator,
        widget=deform.widget.TextInputWidget(template="email.pt"),
    )
    password = colander.SchemaNode(
        colander.String(),
        widget=deform.widget.PasswordWidget(template="viewable_password.pt"),
    )

    @classmethod
    def as_form(cls, request, **override):
        settings = {
            "buttons": ("Create Account", "Cancel"),
            "css_class": "form-horizontal registration",
        }
        settings.update(override)
        return super().as_form(request, **settings)


class ForgottenPasswordSchema(CSRFSchema):
    email_or_username = colander.SchemaNode(colander.String())

    @classmethod
    def as_form(cls, request, **override):
        settings = {
            "buttons": ("Send Reset Link", "Cancel"),
            "css_class": "form-horizontal registration",
        }
        settings.update(override)
        return super().as_form(request, **settings)


class ResetPasswordSchema(CSRFSchema):
    new_password = colander.SchemaNode(
        colander.String(),
        widget=deform.widget.PasswordWidget(template="viewable_password.pt"),
    )

    @classmethod
    def as_form(cls, request, token, **override):
        url = request.resource_url(
            request.context, request.view_name, query={"t": token}
        )
        settings = {
            "buttons": ("Reset Password", "Cancel"),
            "css_class": "form-horizontal registration",
            "url": url,
        }
        settings.update(override)
        return super().as_form(request, **settings)


class MyAccountSchema(CSRFSchema):
    """edit the own account"""

    user_name = colander.SchemaNode(
        colander.String(),
        widget=deform.widget.TextInputWidget(
            template="textinput_disabled.pt", css_class="o3-reg-username"
        ),
    )
    first_name = colander.SchemaNode(colander.String())
    last_name = colander.SchemaNode(colander.String())
    email = colander.SchemaNode(
        colander.String(),
        validator=colander.Email(),
        widget=deform.widget.TextInputWidget(template="email.pt"),
    )

    @classmethod
    def as_form(cls, request, **override):
        settings = {
            "buttons": ("Save Changes", "Cancel"),
            "css_class": "form-horizontal registration",
        }
        settings.update(override)
        return super().as_form(request, **settings)


class EditAccountSchema(CSRFSchema):
    """edit an account"""

    user_name = colander.SchemaNode(
        colander.String(),
        widget=deform.widget.TextInputWidget(
            template="textinput_disabled.pt", css_class="o3-reg-username"
        ),
    )
    role = colander.SchemaNode(
        colander.String(),
        widget=deform.widget.SelectWidget(
            values=ROLES, css_class="custom-select"
        ),
    )
    first_name = colander.SchemaNode(colander.String())
    last_name = colander.SchemaNode(colander.String())
    email = colander.SchemaNode(
        colander.String(),
        validator=colander.Email(),
        widget=deform.widget.TextInputWidget(template="email.pt"),
    )

    @classmethod
    def as_form(cls, request, **override):
        settings = {
            "buttons": ("Save Changes", "Cancel"),
            "css_class": "form-horizontal registration",
        }
        settings.update(override)
        return super().as_form(request, **settings)
