"""User Authentication and Authorization"""

from passlib.context import CryptContext
from pyramid.authentication import AuthTktCookieHelper
from pyramid.authorization import ACLHelper, Authenticated, Everyone
from sqlalchemy.orm.exc import NoResultFound


class SecurityPolicy:
    def __init__(self, secret):
        self.helper = AuthTktCookieHelper(secret)

    def identity(self, request):
        # define our simple identity as None or a dict with
        # userid and principals keys
        identity = self.helper.identify(request)
        if identity is None:
            return None
        userid = identity[
            "userid"
        ]  # identical to the deprecated request.unauthenticated_userid

        # verify the userid, just like we did before with groupfinder
        try:
            user = request.repo.get_user(userid)
            if not user.is_active:
                return None
        except NoResultFound:
            return None

        # assuming the userid is valid, return a map with userid and principals
        return {
            "userid": user.id,
            "principals": user.principals,
        }

    def authenticated_userid(self, request):
        # defer to the identity logic to determine if the user id logged in
        # and return None if they are not
        identity = request.identity
        try:
            user = request.repo.get_user(identity["userid"])
        except (NoResultFound, TypeError):
            return None
        else:
            return user.id

    def permits(self, request, context, permission):
        # use the identity to build a list of principals, and pass them
        # to the ACLHelper to determine allowed/denied
        identity = request.identity
        principals = {Everyone}
        if identity is not None:
            principals.add(Authenticated)
            principals.update(identity["principals"])
        return ACLHelper().permits(context, principals, permission)

    def remember(self, request, userid, **kw):
        return self.helper.remember(request, userid, **kw)

    def forget(self, request, **kw):
        return self.helper.forget(request, **kw)


def get_user(request):
    """retrieves the user object by the unauthenticated user id"""
    identity = request.identity
    if identity is None:
        return None
    try:
        user = request.repo.get_user(identity["userid"])
    except (NoResultFound, TypeError):
        return None
    else:
        return user if user.is_active else None


def get_passlib_context():
    """configures a passlib context and returns it"""
    ctx = CryptContext(
        schemes=["argon2", "bcrypt"], default="argon2", deprecated=["bcrypt"]
    )
    return ctx


def includeme(config):
    """initializing authentication and authorization for the Pyramid app

    Activate this setup using ``config.include('ordr2.security')``.
    """
    settings = config.get_settings()
    policy = SecurityPolicy(settings["auth.secret"])
    config.set_security_policy(policy)
    config.add_request_method(get_user, "user", reify=True)
